#!/usr/bin/perl -w

use strict;
use warnings;

use FileHandle;
use Getopt::Long;

our($opt_rewrite, $opt_ftr, $opt_src, $opt_unique, $opt_duphint, $opt_h, $opt_help) = (0, "", "", 0, "", 0, 0);
Getopt::Long::GetOptions("ftr=s", "src=s", "rewrite", "unique", "duphint", "h", "help");

&usage if (($opt_ftr ne "" && $opt_unique == 1) || ($opt_ftr eq "" && $opt_unique == 0) || $opt_src eq "" || $opt_h == 1 || $opt_help == 1);


select STDERR; $| = 1;
select STDOUT; $| = 1;

my $list = new FileHandle($opt_src);
my $output = new FileHandle("> $opt_src-filtered");
my @filter;

# create the filter array -- either via a file read or via working through 
# all the defined roms and weeding out duplicates
if ($opt_unique == 1) {
   my @duphints;

   if (-f $opt_duphint) {
      my $fh = new FileHandle($opt_duphint);
      die "Cannot open duphint file $opt_duphint: $!\n" if (! defined $fh);
      @duphints = <$fh>;
      chomp(@duphints);
      close $fh;
   }

   # first pass through the list
   my $firstline;
   my @tmproms;
   while ($firstline = <$list>) {
      push @tmproms, $firstline;
   
      # get the next 12 lines
      for my $i (1..12) {
         my $line = <$list>;
      }
   }
   
   # rewind
   seek($list, 0, 0);

   # sort romnames
   my @roms = sort @tmproms;

   # run through the list throwing out duplicates
   my $romi = 0;
   while ($romi <= $#roms) {
      my @matches;
      push @matches, $roms[$romi];
      my $romNameTruncated;
      ($romNameTruncated = $roms[$romi]) =~ s/ \(.*$//;
      $romNameTruncated =~ s/ \[.*$//;

      # find all matches possible
      $romi++;
      if ($romi <= $#roms) {
         while (1) {
            my $thisNameTruncated;
            ($thisNameTruncated = $roms[$romi]) =~ s/ \(.*$//;
            $thisNameTruncated =~ s/ \[.*$//;
   
            # we're done if we find a non-matching entry
            last if ($romNameTruncated ne $thisNameTruncated);
   
            push @matches,$roms[$romi];
   
            $romi++;
         }
      }

      # check for hint
      my $romToUse = "";
      foreach my $hint (@duphints) {
         my @line = split /=/,$hint;
         if ($romNameTruncated eq $line[0]) {
            $romToUse = $line[1];
            last;
         }
      }

      if ($romToUse ne "") {
         push @filter, $romToUse;
      } elsif ($#matches > 0) {
         push @filter, pickFromDuplicates(@matches);
      } else {
         push @filter, $matches[0];
      }
   }

} else {

   my $filter = new FileHandle($opt_ftr);
   die "Cannot open filter file $opt_ftr: $!\n" if (! defined $filter);
   # read in the filter
   @filter = <$filter>;
   close $filter;
}


# loop over the whole input list, one block of info at a time
my $firstline;
my @lines;
while ($firstline = <$list>) {
   $lines[0] = $firstline;

   # get the next 12 lines
   for my $i (1..12) {
      $lines[$i] = <$list>;
   }

   # is this game in the filter?
   if (grep(((uc($_) eq uc($lines[0])),@filter))) {
      for my $j (0..12) {
         print $output $lines[$j];
      }
   }
}

close $list;
close $output;

if ($opt_rewrite) {
   unlink $opt_src;
   rename "$opt_src-filtered", $opt_src;
}



sub pickFromDuplicates {
   my (@dups) = (@_);

   # rule 1: throw out any that have PAL
   my @set1 = grep !/\(pal.*\)/i,@dups;
   if ($#set1 == 0) {
#     print "rule 1:$set1[0]\n\n";
      return $set1[0];
   } elsif ($#set1 < 0) {
      @set1 = @dups;
   }
 

   # rule 2: throw out any prototypes
   my @set2 = grep !/prototype/i,@set1;
   if ($#set2 == 0) {
#     print "rule 2:$set2[0]\n\n";
      return $set2[0];
   } elsif ($#set2 < 0) {
      @set2 = @set1;
   }


   # rule 3: single [!]?
   @set1 = grep /\[!\]/,@set2;
   if ($#set1 == 0) {
#     print "rule 3:$set1[0]\n\n";
      return $set1[0];

   # rule 4: shortest of the [!]'s
   } elsif ($#set1 > 0) {
      my $len = 999999;
      my $chooseme = "";
      foreach my $l (@set1) {
         if (length($l) < $len) {
            $len = length($l);
            $chooseme = $l;
         }
      }

#     print "rule 4:$chooseme\n\n";
      return $chooseme;
   }
   
   # rule 5: choose first from those left after above
   if ($#set2 > 0) {
#     print "rule 5:$set2[0]\n\n";
      return $set2[0];
   }


   # last resort, just pick the first one
#  print "dups=\n";
#  for (my $i=0; $i <= $#dups; $i++) {
#     print "  $dups[$i]\n";
#  }
#  print "rule LAST:$dups[0]\n\n";

   return $dups[0];

#
#  Some ideas for picking:
#
#  1. throw out any that say PAL
#  2. throw out any that say "PROTOTYPE"
#  3. choose [!] version if/when there's only one of them
#  4. choose the shortest named [!] version when there's more than one
#  5. choose the first one of the ones that are left
#  L. choose the first one out of what was passed down
}



sub usage {
print <<EOHELP;
filter-mamewah-list.pl [options] {-ftr <filter file> | -unique} -src <MAMEWah source list>

Takes a MAMEWah list and pulls out only items that match those in the filter 
list.

Options:
   -unique
      Remove duplicate entries in the game list by comparing game names.

   -rewrite
      Rename the output list to that of the MAMEWah source list upon finish.

   -duphints <hints file>
      A file of "hints" that tell the script which specific rom to choose 
      when it has to choose between alternatives.

EOHELP
exit 1;
}
