#!/usr/bin/perl

use strict;
use warnings;

use FileHandle;
use Getopt::Long;
use Data::Dumper;

our ($opt_inis, $opt_uniqifier, $opt_h, $opt_help) = ("","",0,0);

Getopt::Long::GetOptions("inis=s", "uniqifier=s","h","help");

&usage if ($opt_h || $opt_help);
&usage if ($#ARGV != 2);
&usage if ((! -f $ARGV[0]) || (! -f $ARGV[1]));

select STDERR; $| = 1;
select STDOUT; $| = 1;

my %list1;
my %list2;

open FH1, "$ARGV[0]" or die "Cannot open $ARGV[0]: $!\n";
my $line;
while ($line = <FH1>) {
   chomp($line);
   $list1{$line} = [];
   for my $i (0 .. 11) {
      $list1{$line}[$i] = <FH1>;
      chomp($list1{$line}[$i]);
   }
}
close FH1;

open FH2, "$ARGV[1]" or die "Cannot open $ARGV[1]: $!\n";
while ($line = <FH2>) {
   chomp($line);
   $list2{$line} = [];
   for my $i (0 .. 11) {
      $list2{$line}[$i] = <FH2>;
      chomp($list2{$line}[$i]);
   }
}
close FH2;

my @ini;

if ($opt_inis ne "") {
   open FHINI, $opt_inis or die "Cannot open .ini template file $opt_inis: $!\n";
   @ini = <FHINI>;
   chomp(@ini);
   close FHINI;
}


for my $romname (keys %list2) {
   if (grep(($_ eq $romname),(keys %list1))) {
      # game on list2 that is already in list1, process it
      if ($opt_uniqifier ne "") {
         # we need to uniqify and add this
         $list1{"$opt_uniqifier$romname"} = $list2{$romname};
         make_list2_ini_file($romname,$opt_uniqifier) if ($opt_inis ne "");
      }
      # else we don't need to add it
      
   } else {
      # game on list2 not on list1, add it & make .ini file
      $list1{$romname} = $list2{$romname};
      make_list2_ini_file($romname) if ($opt_inis ne "");
   }
}

# order keys by game name
my @keys = sort {
   return $list1{$a}[0] cmp $list1{$b}[0];
} (keys %list1);

open FHW, "> $ARGV[2]" or die "Cannot open output list $ARGV[2]: $!\n";
for my $key (@keys) {
   my $ar = $list1{$key};
   print FHW "$key\n";
   for my $i (@$ar) {
      print FHW "$i\n";
   }
}
close FHW;


sub make_list2_ini_file {
   my $game = shift @_;
   my $uniq = shift @_;

   my $gamename = $game;

   $gamename = "$uniq$game" if (defined $uniq);

   my $fname = lc($gamename);

   open FH, "> $fname.ini" or die "Cannot open $fname.ini for writing: $!\n";

   for my $line (@ini) {
      # the $l temp var is necessary, because we don't want to change the 
      # @ini array itself
      my $l = $line;
      $l =~ s/\[ini_gamename\]/$game/ig;
      $l =~ s/\[ini_unique_gamename\]/$gamename/ig;
      print FH "$l\n";
   }

   close FH;
}


sub usage {
print <<EOUSAGE;
merge-mamewah-lists.pl [options] <MAMEWah list1> <MAMEWah list2> <output list>

Merges two MAMEWah lists into a third.

Options:
   -uniqifier <string>
      Change the name of any game that is on both lists by adding <string> to 
      the front.  When creating game-specific .ini files, the uniqified name 
      will be used for the .ini filename.

   -inis <ini file template filename>
      Create a set of game-specific .ini files for the games on list2 using 
      the file specified as a template.
      
      The string [ini_gamename] will be replaced with the rom name anywhere 
      it exists in the template.

      The string [ini_unique_gamename] will be replaced with the rom name as 
      uniqified anywhere it exists in the template.

EOUSAGE
exit 1;
}
